// See COPYING for license information

#include <algorithm>
#include <fstream>
#include "defs.h"
#include "CParser.h"
#include "CDownloader.h"
#include "CConfigFile.h"
#include "CPkgGetter.h"

CPkgGetter::CPkgGetter(std::vector<std::string>& packages, const bool& autodeps)
{
	// All packages must be lower case to start with.
	for ( unsigned int i = 0; i < packages.size(); i++ )
		std::transform(packages[i].begin(), packages[i].end(), packages[i].begin(), tolower);

	// Find the some configuration parameters.
	std::string	_mirror =	CConfigFile::Instance().GetValue("CurrentMirror")[0];
	std::string	_cache =	CConfigFile::Instance().GetValue("CacheDir")[0];
	std::string	_blacklist =	CConfigFile::Instance().GetValue("BlackList")[0];
	std::transform(_blacklist.begin(), _blacklist.end(), _blacklist.begin(), tolower);

	// Load up the package DB.
	CParser		_parser;
	package_map&	_db = _parser.Packages;

	add_dependencies(_db, packages, autodeps);

	// Chick chiggity check the blacklist.
	// Also check for packages that are about to be reinstalled.
	for ( unsigned int i = 0; i < packages.size(); i++ ) {
		if ( (int)_blacklist.find("," + packages[i] + ",") >= 0 )
			std::cerr << VT_COLOR_DOWNGRADE << "Package '" << _db[packages[i]].name << "' is on the blacklist, skipping.\n" << VT_COLOR_NORMAL;
		else if ( _db[packages[i]].mirror_version == _db[packages[i]].installed_version &&
				_db[packages[i]].mirror_version.length() && !reinstall_prompt(_db[packages[i]].name) );
		else	continue;
		packages.erase(packages.begin() + i);
		i--;
	}

	// Snag those files!
	CDownloader	_dler;
	for ( unsigned int i = 0; i < packages.size(); i++ ) {
		std::string	_pkg = "/" + _db[packages[i]].location + "/" + _db[packages[i]].name + "-" + _db[packages[i]].mirror_version + _db[packages[i]].type;
		if ( file_exists(_cache + _pkg) )
			std::cout << "Package '" << _db[packages[i]].name << "-" << _db[packages[i]].mirror_version << "' found in cache.\n";
		else if ( _db[packages[i]].mirror_version.empty() ) {
			std::cerr << VT_COLOR_DOWNGRADE << "Package '" << packages[i] << "' not found on mirror, skipping.\n" << VT_COLOR_NORMAL;
			packages.erase(packages.begin() + i);
			i--;
			continue;
		} else {
			std::cout << "Downloading (" << i + 1 << " of " << packages.size() << "): " << _mirror + _pkg << std::endl;
			if ( !_dler.Download(_mirror + _pkg, _cache + _pkg) ) {
				std::cout << VT_COLOR_DOWNGRADE <<  "Package '" << _pkg << "' failed to download.\n" << VT_COLOR_NORMAL;
				packages.erase(packages.begin() + i);
				i--;
				continue;
			}
		}
		packages[i] = "cache://" + _cache + _pkg;
	}
}

void CPkgGetter::add_dependencies(package_map& db, std::vector<std::string>& packages, const bool& autodeps)
{	// Add all dependencies and subdependencies.
	for ( unsigned int i = 0; i < packages.size(); i++ ) {
		std::string	_strdeps = db[packages[i]].dependencies;
		if ( !_strdeps.empty() )	_strdeps = _strdeps.substr(1);	// Chop off prefix comma.
		std::transform(_strdeps.begin(), _strdeps.end(), _strdeps.begin(), tolower);

		while ( (int)_strdeps.find(',') > 0 ) {
			std::string	_pkg = _strdeps.substr(0, _strdeps.find(','));
			std::transform(_pkg.begin(), _pkg.end(), _pkg.begin(), tolower);
			_strdeps = _strdeps.substr(_strdeps.find(',') + 1);

			if ( db.find(_pkg) != db.end() && db[_pkg].installed_version.empty() &&
					!db[_pkg].mirror_version.empty() ) {
				bool	_found = false;
				for ( unsigned int k = 0; k < packages.size(); k++ )
					if ( packages[k] == _pkg )	_found = true;
				if ( !_found && dep_prompt(db[packages[i]].name, db[_pkg].name, autodeps) )	packages.push_back(_pkg);
			}
		}
	}
}

bool CPkgGetter::reinstall_prompt(const std::string& package)
{	// Make sure the user wants to reinstall with the same version again.
	std::cout << "You are attempting to reinstall '" << package << "'.\n";

	std::string	_choice;
	while ( _choice.find('y') && _choice.find('n') ) {
		std::cout << "Are you sure you want to reinstall the package? [y/n] ";
		std::cout.flush();
		std::getline(std::cin, _choice);
	}
	return _choice.find('n');
}

bool CPkgGetter::dep_prompt(const std::string& package, const std::string& dep, const bool& autodeps)
{	// Prompts the user about whether or not to install some dep.
	if ( autodeps )	return true;	// Automatically say yes.
	std::cout << "Package '" << package << "' requires dependency '" << dep << "'.\n";

	std::string	_input;
	while ( _input.find('y') && _input.find('n') ) {
		std::cout << "Would you like to install the dependency? [y/n] ";
		std::cout.flush();
		std::getline(std::cin, _input);
	}
	return _input.find('n');
}

bool CPkgGetter::file_exists(const std::string& filename)
{	// Determines whether or not a file exists.
	std::ifstream	_iftest;
	_iftest.open(filename.c_str(), std::ios_base::in);
	if ( !_iftest.is_open() )	return false;
	_iftest.close();
	return true;
}
