// See COPYING for license information.

#include <algorithm>
#include <time.h>
#include <sys/stat.h>
#include "defs.h"
#include "CMirrorHandler.h"
#include "CVersionCompare.h"
#include "CPkgLister.h"

#define MAX(arg1, arg2)		(arg1 > arg2 ? arg1 : arg2)
// This macro pads a string with tabs.
#define PAD_TABS(str, numtabs)	str += std::string(MAX(numtabs - (int)(str.length() / 8), 0), '\t')
// Pads a string with spaces.
#define PAD_SPACES(str, numspaces)	str += std::string(MAX(numspaces - str.length(), 0), ' ')

CPkgLister::CPkgLister(const std::string& filter, std::vector<std::string>& queries)
{
	reload_old_mirror();

	// Read the package DB and display it.
	CParser		_parser;
	package_map&	_packages = _parser.Packages;
	package_map	_dest;
	unsigned int	_long_cat = 0, _long_name = 0, _long_version = 0;

	// Convert any queries to lower case.
	for ( unsigned int i = 0; i < queries.size(); i++ )
		std::transform(queries[i].begin(), queries[i].end(), queries[i].begin(), tolower);

	for ( package_map::iterator _iter = _packages.begin(); _iter != _packages.end(); ++_iter ) {
		// Find the longest category, name, and mirror version
		// so that formatting can be more awesome.
		if ( _iter->second.location.length() > _long_cat )		_long_cat = _iter->second.location.length();
		if ( _iter->first.length() > _long_name )			_long_name = _iter->first.length();
		if ( _iter->second.mirror_version.length() > _long_version )	_long_version = _iter->second.mirror_version.length();

		// Skip displaying this package if it doesn't fit the filter.
		bool	_skippage = true;
		if ( (int)filter.find('a') >= 0 )
			_skippage = false;
		else if ( (int)filter.find('u') >= 0 && _iter->second.mirror_version != _iter->second.installed_version &&
				!_iter->second.installed_version.empty() && !_iter->second.mirror_version.empty() )
			_skippage = false;
		else if ( (int)filter.find('n') >= 0 && _iter->second.installed_version.empty() && !_iter->second.mirror_version.empty() )
			_skippage = false;
		else if ( (int)filter.find('i') >= 0 && !_iter->second.installed_version.empty() )
			_skippage = false;
		else if ( (int)filter.find('m') >= 0 && !_iter->second.mirror_version.empty() )
			_skippage = false;
		else if ( filter.empty() && matches_query(_iter->first, queries) )
			_skippage = false;
		if ( _skippage || _iter->first.empty() )	// You fail, don't display this.
			continue;

		_dest[_iter->first] = _iter->second;
	}
	
	display(_dest, _long_cat + 2, _long_name, _long_version + 3);
}

void CPkgLister::reload_old_mirror()
{	// Reload mirror if the PACKAGES.TXT file is old.
	struct stat	_dbstats;
	stat(TMPFILE("PACKAGES.TXT").c_str(), &_dbstats);
	if ( time(NULL) - _dbstats.st_mtime > PACKAGES_TXT_MAX_AGE )	CMirrorHandler(false);
}

bool CPkgLister::matches_query(const std::string& name, const std::vector<std::string>& queries)
{
	if ( !queries.size() ) {
		std::cerr << "You must specify at least one pattern to be matched\n";
		std::cerr << "with any and all package names.\n";
		exit(1);
	}

	// Convert to lower case for case insensitive comparison.
	std::string	_lower_name = name;
	std::transform(_lower_name.begin(), _lower_name.end(), _lower_name.begin(), tolower);

	for ( unsigned int i = 0; i < queries.size(); i++ )
		if ( _lower_name.find(queries[i]) <= _lower_name.length() )	return true;

	return false;
}

void CPkgLister::display(package_map& packages, const unsigned int& cat, const unsigned int& name, const unsigned int& version)
{	// Display the packages we read from the DB.
	CVersionCompare				_vcomp;
	for ( std::map<std::string, Spackage_simple>::iterator _iter = packages.begin(); _iter != packages.end(); ++_iter ) {
		// Choose the correct color for this package.
		std::string	_location = _iter->second.location;
		std::string	_pkg = _iter->second.name;
		std::string	_mirror_version = _iter->second.mirror_version;
		if ( _mirror_version.empty() || _iter->second.installed_version.empty() )
			std::cout << VT_COLOR_NORMAL;
		else if ( _vcomp.Compare(_mirror_version, _iter->second.installed_version) )
			std::cout << VT_COLOR_UPGRADE;
		else if ( _vcomp.Compare(_iter->second.installed_version, _mirror_version) )
			std::cout << VT_COLOR_DOWNGRADE;
		else	std::cout << VT_COLOR_NORMAL;

		// Add whitespace for formatting, and display the package.
		_location = "[" + _location + "]";
		PAD_SPACES(_location, cat + 1);
		PAD_SPACES(_pkg, name + 1);
		_mirror_version = "M: " + _mirror_version;
		PAD_SPACES(_mirror_version, version + 1);
		_iter->second.installed_version = "I: " + _iter->second.installed_version;
		std::cout << _location << _pkg << _mirror_version << _iter->second.installed_version << VT_COLOR_NORMAL << std::endl;
	}
}
